;-----------------------------------------------------------------------
;
; Definition file for the YM2149 emulator II.
;
; 1st version	22/01/1997
; v0.91		16/06/97	PanelMoveSprites
;
;-----------------------------------------------------------------------

YM2149_I	SET	1

	incdir	include:

	IFND	EXEC_TYPES_I
	include	"exec/types.i"
	ENDC

	IFND UTILITY_TAGITEM_I
	    INCLUDE "utility/tagitem.i"
	ENDC



		; Offsets of the available routines. The main binary contains
		; some bra.w to each of these routines.

		; Main Routines
YM_EmulatorHardInit	=	0
YM_EmulatorHardEnd	=	4
YM_EmulatorOSInit	=	8
YM_EmulatorOSEnd	=	12
YM_EmulatorInit		=	16
YM_Emulator		=	20

		; Panel Routines
YM_PanelInit		=	24
YM_PanelPutSprites	=	28
YM_Panel		=	32

		; Utility Routines
YM_EmulatorDebug	=	36
YM_4bitTo8bit		=	40
YM_4bitTo8bitLinear	=	44
YM_SignedSample		=	48

		; ym2149.s v0.91
YM_PanelMoveSprites	=	52


		; Tags and macro used to define the header of a file
		; supported by the YM-2149 player.

YM_FILE_IDENT		=	"YMST"	; 1st bytes of a valid file


YMT_Base		=	TAG_USER+"YM"

	ENUM	YMT_Base		; TagBase

	EITEM	YMT_Type		; ext replay, int replay, music data
	EITEM	YMT_Version		; version for this file
	EITEM	YMT_ExtReplayName	; if type = music data
	EITEM	YMT_EmulatorData	; will contain the address of YM_GlobalData

	EITEM	YMT_MiscInfo		; just for info purpose, not used by emulator
	EITEM	YMT_DurationArray	; pointer to array of long words
					; high word=minutes  low word=seconds

	EITEM	YMT_FileName		; pointer to name
	EITEM	YMT_AuthorName		; ptr to author
	EITEM	YMT_ConverterName	; ptr to adapter/ripper of the music
	EITEM	YMT_SongNameArray	; ptr to array of names for subsongs
	EITEM	YMT_DescriptionText	; for extra infos on the file

	EITEM	YMT_MasterInit		; ptr to init rout for both ST/Amiga
					; (called only one time)
	EITEM	YMT_MasterPatch		; Amiga specific patches for emulation
	EITEM	YMT_SongInit		; init one subsong
	EITEM	YMT_Replay		; routine called each VBL on both ST/Amiga
	EITEM	YMT_Emulate		; Amiga specific, pass YM values to emulator

	EITEM	YMT_MinSong		; 1 ..
	EITEM	YMT_MaxSong		; 1 ..
	EITEM	YMT_FirstSong		; subsong to be played at start
	EITEM	YMT_SongNbrArray	; ptr to array of max-min songs number

	EITEM	YMT_Data		; pointer to the original start of
	EITEM	YMT_DataLen		; the music data.

	EITEM	YMT_ExtDataAdr		; start of data for external replay
	EITEM	YMT_ExtDataLen		; len

	EITEM	YMT_VoicesPref		; ptr to values for mapping ST voices
					; to the Amiga.


		; Values for YMT_MiscInfo
YMT_MISC_HANDLE_ENV	=	1	; replay handles enveloppe
YMT_MISC_USE_ENV	=	2	; music data handles and uses enveloppe
YMT_MISC_USE_CUSTOM	=	4	; music uses a custom enveloppe
YMT_MISC_ABSCODE	=	8	; replay needs to be relocated
YMT_MISC_USE_SAMPLE	=	16	; music uses samples


		; Values for YMT_Type
YMT_INT_REPLAY		=	0	; replay+data in the same file
YMT_EXT_REPLAY		=	1	; replay only
YMT_MUS_DATA		=	2	; data only (YMT_ExtReplayName required)


		; This macro defines the beginning of a file

YM_FILE_HEADER	MACRO
	IFND	YM_DEBUG
	moveq	#-1,d0			; return an error if one tries
	rts				; to run the file
	ELSEIF
	lea	YM_FILE_START(pc),a0
	jmp	YM_EmulatorBin+YM_EmulatorDebug
	ENDC

YM_FILE_START				; label used by YM_TAGP macro
	dc.l	YM_FILE_IDENT		; "YMST"
	; Following values are YM tag values
	ENDM



		; This macro defines the end of a file

YM_FILE_END	MACRO
	IFD	YM_DEBUG
	EVEN
YM_EmulatorBin
	incbin	asm:YM2149/YM2149.bin
	ENDC
	ENDM


		; Tags' definition macros

YM_TAG		MACRO			; to define tag with integer value
	dc.l	\1,\2
	ENDM

YM_TAGP		MACRO			; to define tag with pointer value
	dc.l	\1,\2-YM_FILE_START	; write the pointer as a relative
	ENDM				; value




		; The structure containing the copy of the values
		; passed to the YM2149 by the ST's replay.

 STRUCTURE	YM_STData_struct,0	; order should never be changed
	UWORD	YM_VoiceAPer		; 0 a fff
	UWORD	YM_VoiceAVol		; 0 a 1f
	UWORD	YM_VoiceBPer		; 0 a fff
	UWORD	YM_VoiceBVol		; 0 a 1f
	UWORD	YM_VoiceCPer	 	; 0 a fff
	UWORD	YM_VoiceCVol		; 0 a 1f
	UWORD	YM_NoisePer		; 0 a 1f
	UWORD	YM_InUse		; 0 a 3f ( bit=0 <=> canal ON )
	UWORD	YM_EnvWave		; 0 a f  ou <0
	UWORD	YM_EnvPer		; 0 a ffff

	LABEL	YM_STData_size



		; The structure describing the info associated with
		; one ST's voice.

 STRUCTURE	YM_InfoVoice_struct,0
	UWORD	YM_Status		; type of output (copro, sample, ...)
	UWORD	YM_CustomEnv		; 0=off
	UWORD	YM_CustomEnvLen		; in bytes, must be a power of 2
	UWORD	YM_CustomEnvPer		; same meaning as 'normal' env
	ULONG	YM_CustomEnvParam	; depends on the custom env nbr
	ULONG	YM_LastNoiseIndex
	ULONG	YM_LastEnvIndex
	APTR	YM_LastEnvAdr		; 0=no env at the moment
	ULONG	YM_AudioBase		; DFF0A0|B0|C0|D0
	UWORD	YM_AudioDMA		; 1, 2, 4 or 8
	UWORD	YM_MaxVolDMA		; 0 .. 64
	APTR	YM_SmpStart
	UWORD	YM_SmpLen
	UWORD	YM_SmpPer
	UWORD	YM_SmpVol
	APTR	YM_SmpNextStart
	UWORD	YM_SmpNextLen
	UWORD	YM_SmpNextPer
	UWORD	YM_SmpNextVol

	LABEL	YM_IV_size



		; The global structure, contains all variable datas

 STRUCTURE	YM_Main_struct,0
	STRUCT	YM_STData,YM_STData_size; always at the beginning of Main_struct

	APTR	YM_EmulatorBase		; pointer to start of bra's table.

	UWORD	YM_DebugColor		; 0=colors OFF, else ON

	UWORD	YM_DMA_Output		; which DMA voices to use on Amiga
	UWORD	YM_Map_Copy		; which of the 3 ST's voice should be
					; duplicated as the 4th voice on Amiga

	STRUCT	YM_InfoVoiceA,YM_IV_size	; info for ST VoiceA
	STRUCT	YM_InfoVoiceB,YM_IV_size	; info for ST VoiceB
	STRUCT	YM_InfoVoiceC,YM_IV_size	; info for ST VoiceC
	STRUCT	YM_InfoVoiceCopy,YM_IV_size	; copy of A, B or C

	APTR	YM_DMAVoice0	; pointer to InfoVoice associated to DMA 0
	APTR	YM_DMAVoice1
	APTR	YM_DMAVoice2
	APTR	YM_DMAVoice3

	APTR	YM_ST_AMvol		; 16 bytes table

	APTR	YM_SampleABuffer1	; pointer to 600 bytes in CHIP
	APTR	YM_SampleABuffer2	; uses double buffer
	APTR	YM_SampleBBuffer1
	APTR	YM_SampleBBuffer2	
	APTR	YM_SampleCBuffer1
	APTR	YM_SampleCBuffer2	
	UWORD	YM_SampleBufferIndex	; 0 or 4

	APTR	YM_SampleNoise		; random sample (=noise)
	APTR	YM_SampleNoise_7f	; ( random sample + $7f ) / 2
	APTR	YM_SampleNoise_80	; ( random sample - $7f ) / 2
	UWORD	YM_SampleNoiseLen

	APTR	YM_SampleEmpty		; empty sample, len=1 word

	APTR	YM_SampleCopro2		; pattern len=2 bytes
	APTR	YM_SampleCopro4		; pattern len=4 bytes
	APTR	YM_SampleCopro8		; pattern len=8 bytes
	APTR	YM_SampleCopro16	; pattern len=16 bytes
	APTR	YM_SampleCopro32	; pattern len=32 bytes
	APTR	YM_SampleCopro64	; pattern len=64 bytes
	APTR	YM_SampleCopro128	; pattern len=128 bytes
	APTR	YM_SampleCopro256	; pattern len=256 bytes

	UWORD	YM_MinPer		; if Per<MinPer -> Voice Off

	LABEL	YM_MS_size



		; Possible type of custom enveloppe.

YM_NO_CUSTOM_ENV	=	0
YM_SQUARE_ENV		=	1
YM_OWN_ENV		=	2


		; The different type of samples for the emulation.
		; Values with a '*' require mixing techniques.

YM_EMPTY		=	0	; no sound
YM_COPRO		=	1	; square wave ($7f $81)
YM_NOISE		=	2	; noise
YM_COPRO_NOISE		=	4	; square wave + noise		(*)
YM_COPRO_ENV		=	8	; square wave + env		(*)
YM_NOISE_ENV		=	16	; noise + env			(*)
YM_COPRO_NOISE_ENV	=	32	; square + noise + env		(*)
YM_ENV			=	64	; env alone (special case)
YM_SAMPLE		=	128	; sample sets by user (drums, etc.)
YM_NEW_PERVOL		=	$4000	; immediate change of Per/Vol
YM_NEW_SOUND		=	$8000	; start a new type of sample



		; To decide which ST's voice to duplicate

YM_NO_MAP		=	-1	; don't duplicate any voice
YM_MAP_A		=	0	; duplicate voice A
YM_MAP_B		=	1	; duplicate voice B
YM_MAP_C		=	2	; duplicate voice C



		; Some more constants

YM_EmulFactor	=	58647	; 14.3181*4096
YM_DMAwait	=	6	; 3 number of raster lines between DMA OFF/ON

YM_VoiceSampleLen =	512	; max used : 512 bytes/vbl -> 25600 Hz
YM_NoiseLen	=	4096	; bytes



;-----------------------------------------------------------------------
;			end of definitions file
;-----------------------------------------------------------------------
